/* =================================================================================
 *  alberi.c
 * =================================================================================
 * Contiene l'implementazione di funzioni e procedure su alberi binari.
 *
 *  Authors:     Emanuela Fachini  (fachini@di.uniroma1.it )
 *               Irene Finocchi    (finocchi@di.uniroma1.it)
 *               Chiara Petrioli   (petrioli@di.uniroma1.it)
 *  Last change: 14 Aprile 2005
 *  Changed by:  Irene Finocchi
*/

#include "alberi.h"

/* Pre: l'albero che si costruisce e' NON vuoto*/
/* Post: costruisce in preordine un albero binario*/
TreePtr creaalbero() {
	TreePtr T;
	int i,j;

	scanf("%d %d",&i,&j);
	if (i==-1 && j==-1) return NULL;
	T = malloc(sizeof(TREENODE));

	if (T==NULL) printf("Impossibile allocare memoria per l'albero\n"); 
	else {
		T->elem=i;
		switch(j) {
			case 0:  T->rPtr=NULL;
				     T->lPtr=NULL;
				     break;
			case -1: T->lPtr=creaalbero();
				     T->rPtr=NULL;
				     break;
			case 1:  T->lPtr=NULL;
				     T->rPtr=creaalbero();
				     break;
			case 2:  T->lPtr=creaalbero();
				     T->rPtr=creaalbero();
				     break;
		}
	}
	return(T);
}


/* Post: restituisce 1 se l'albero e' vuoto, 0 altrimenti*/
int alberovuoto(TreePtr T) {
	return(T==NULL);
}

/* Post: stampa in inorder i valori contenuti nei nodi dell'albero*/
void stampaAlberoInorder(TreePtr T) {
	if (T!=NULL) {
		stampaAlberoInorder(T->lPtr);
		printf("%d ",T->elem);
		stampaAlberoInorder(T->rPtr);
	}
	return;
}

/* OSSERVAZIONE: nel main chiamata con h a zero */
/* Post: stampa i valori dell'albero indentati  */
void stampaIndentata (TreePtr T, int h) {
	int i;
	if (T!=NULL) {
		stampaIndentata(T->rPtr,h+1);
		for (i=1;i<=h;i++)
			printf("\t");
		printf("%d \n", T->elem);
		stampaIndentata(T->lPtr,h+1);
	}
	
}


/* ESERCIZIO 6 */

/* Post: restituisce 1 se U e' un prefisso di T, 0 altrimenti */
int prefTree(TreePtr T, TreePtr U) {
    if (!U) return 1;  /* se U e' vuoto e' prefisso di ogni albero */ 
    if (!T) return 0;  /* qui U non e' vuoto ma T si'! */
	
    if ( T->elem != U->elem ) return 0;
    if (prefTree(T->lPtr,U->lPtr))	
    return prefTree(T->rPtr,U->rPtr);
    else return 0;
	
}


/* ESERCIZIO 13 */

int AlberoPienoAus(TreePtr T, int* ris);

/* Post: restituisce 1 se l'albero e' pieno, 0 altrimenti */
int AlberoPieno(TreePtr T) {
    int h, ris = 1;
    h = AlberoPienoAus(T,&ris);
    return ris;
}

/* Post: restituisce l'altezza di T e pone ris a 1 o a 0 a seconda se T e' pieno o no */
int AlberoPienoAus(TreePtr T, int* ris) {
    int hl,hr;
    if (T==NULL) return -1;
    printf("Visito il nodo %d\n",T->elem);
    hl = AlberoPienoAus(T->lPtr,ris);
    hr = AlberoPienoAus(T->rPtr,ris); 
    if (!((*ris) && hl==hr)) *ris = 0;    	
    if (hl>hr) return hl+1; else return hr+1;
}


int AlberoPienoVeloceAus(TreePtr T, TREENODE **nodo, int* ris);

/* Post: restituisce 1 se l'albero e' pieno, 0 altrimenti */
int AlberoPienoVeloce(TreePtr T, TREENODE **nodo, int *h) {
    int ris = 1;
    *nodo = NULL;
    *h = AlberoPienoVeloceAus(T,nodo,&ris);
    return ris;
}

/* Post: restituisce l'altezza di T se l'albero e' pieno, pone ris a 1 o a 0 a seconda se T e' pieno o no,
   *nodo e' il puntatore al primo nodo sbilanciato incontrato se l'albero non e' pieno */
int AlberoPienoVeloceAus(TreePtr T, TREENODE **nodo, int* ris) {
    int hl,hr;
    if (T==NULL) return -1;
    printf("Visito il nodo %d\n",T->elem);
    hl = AlberoPienoVeloceAus(T->lPtr,nodo,ris);
    if (*ris==0) return hl+1;    	
    hr = AlberoPienoVeloceAus(T->rPtr,nodo,ris); 
    if (*ris==0) return hr+1;
    if (hl!=hr) {
        *ris = 0;
        *nodo = T;
        return 0;
    }
    else return hl+1;
}


/* ESERCIZIO 3 */

TreePtr cerca(TreePtr t, int x);

/* prec: t non contiene ripetizioni                                 */
/* postc: restituisce 1 se c'e' un cammino padri-figli che porta    */
/* da un nodo di campo elem x a uno di campo elem y, 0 altrimenti.  */
int path(TreePtr t, int x, int y) {
    if (cerca(cerca(t,x),y)) return 1; 
    else return 0;
}

/* Prec: nessuna */
/* Postc: restituisce il puntatore al primo nodo dell'albero con valore x, NULL se non esiste */
TreePtr cerca(TreePtr t, int x) {
    TreePtr tempPtr;
    if (!t) return NULL;
    if (t->elem == x) return t;
    tempPtr = cerca(t->lPtr,x);
    if (!tempPtr) return cerca(t-> rPtr,x);
    else return tempPtr;
}

/* ESERCIZIO 3 con ripetizioni di nodi */

TreePtr cerca2(TreePtr t, TreePtr *ptrX, int x, int y);

/* Prec: nessuna                                                     */
/* Postc: restituisce 1 se c'e' un cammino padri-figli che porta     */
/* da un nodo con valore x a un nodo con valore y, 0 altrimenti.     */
int path2(TreePtr t, int x, int y) {
    TreePtr ptrX=NULL;
    TreePtr ptrY = cerca2(t,&ptrX,x,y);
    if (ptrX && ptrY) return 1; 
    else return 0;
}

/* Prec: *ptrX!=NULL se nel cammino dalla radice al nodo corrente esiste un nodo con valore =x */
/* Postc: restituisce un puntatore !=NULL sse l'albero t contiene un cammino da un nodo con valore
   x a un nodo con valore y. In tal caso, il puntatore restituito punta al nodo contenente il valore 
   y, mentre *ptrX punta al nodo contenente il valore x */
TreePtr cerca2(TreePtr t, TreePtr *ptrX, int x, int y) { 
    TreePtr temp;
    if (!t) return NULL;
    if ( !(*ptrX) && t->elem==x ) *ptrX=t;
    if (*ptrX) {
        if (t->elem==y) return t;
        temp=cerca2(t->lPtr,ptrX,x,y);
        if (temp!=NULL) return temp;
        else return cerca2(t->rPtr,ptrX,x,y);
    }
    else {
        temp=cerca2(t->lPtr,ptrX,x,y);
        if (*ptrX && temp) return temp;
        *ptrX=NULL;                       /* rende vera la precondizione */
        temp=cerca2(t->rPtr,ptrX,x,y);
        if (*ptrX && temp) return temp;
        *ptrX=NULL;
        return NULL;
    }
}

