/* =================================================================================
 *  alberi.h
 * =================================================================================
 * Contiene la definizione di albero binario (di interi), ed i prototipi di
 * varie funzioni e procedure su alberi binari.
 *
 *  Authors:     Emanuela Fachini  (fachini@di.uniroma1.it )
 *               Irene Finocchi    (finocchi@di.uniroma1.it)
 *               Chiara Petrioli   (petrioli@di.uniroma1.it)
 *  Last change: 14 Aprile 2005
 *  Changed by:  Irene Finocchi
*/

#ifndef ALBERI_H
#define ALBERI_H

#include <stdio.h>
#include<stdlib.h>

struct nodoalbero{
	struct nodoalbero *rPtr;
	int elem;
	struct nodoalbero *lPtr;
};

typedef struct nodoalbero TREENODE;
typedef TREENODE *TreePtr;


/* Costruzione di un albero binario inserito da input. Si assume che gli interi
 * contenuti nei nodi dell'albero siano inseriti da input in preordine
 * Il formato dell'input saranno coppie di interi in cui il primo valore
 * corrisponde al valore da inserire nel nodo, il secondo valore indica il
 * numero dei suoi figli (0 nessun figlio, 1 solo il figlio sinistro, -1 solo
 * il figlio destro, 2 entrambi i figli). I nodi sono inseriti i preordine (prima la
 * radice), quindi i nodi del sottoalbero sinistro, quindi i nodi del sottoalbero
 * destro. Ad esempio:
 * 3 2
 * 1 0
 * 4 2
 * 5 0
 * 7 0
 *
 * costruisce l'albero binario
 *    3
 *  1	 4
 *     5   7
 * La seguente procedura costruisce l'albero binario */
TreePtr creaalbero();

/* Verifica se l'albero e' vuoto */
int alberovuoto(TreePtr);

/* stampa in inorder i valori contenuti nell'albero */
void stampaAlberoInorder(TreePtr);

/* stampa l'albero indentato */
void stampaIndentata(TreePtr, int);

/* Verifica se un albero e' pieno */
int AlberoPieno(TreePtr T);
int AlberoPienoVeloce(TreePtr T, TREENODE **nodo, int *h);

/* Verifica se l'albero U e' un prefisso dell'albero T */
int prefTree(TreePtr T, TreePtr U);

int path(TreePtr t, int x, int y);
int path2(TreePtr t, int x, int y);


#endif
